package com.hzya.frame.plugin.dgx.oa.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.plugin.dgx.oa.entity.*;
import com.hzya.frame.plugin.dgx.oa.enums.ColEventTypeEnum;
import com.hzya.frame.sysnew.application.database.entity.SysApplicationDatabaseEntity;
import com.hzya.frame.sysnew.application.database.service.ISysApplicationDatabaseService;
import com.hzya.frame.sysnew.application.entity.SysApplicationEntity;
import com.hzya.frame.sysnew.application.entity.SysExtensionApiEntity;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 *
 * @content 扩展类 解析参数并重新进行组装
 * @Param
 * @Return
 * @Author hecan
 * @Date 2024-09-23 17:18
 * **/
@Service(value = "ParseAndAssembleService")
public class ParseAndAssembleService {
    Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ISysApplicationDatabaseService sysApplicationDatabaseService;

    public SysExtensionApiEntity  parseAndAssemble(SysExtensionApiEntity entity){
        try {
            String bodys = entity.getBodys();
            String dataSourceCode = "";
            SysApplicationEntity receiveApp = entity.getReceiveApp();
            if (null != receiveApp){
                SysApplicationDatabaseEntity sysDatabaseEntity = new SysApplicationDatabaseEntity();
                sysDatabaseEntity.setAppId(receiveApp.getId());
                List<SysApplicationDatabaseEntity> dataBaseList = sysApplicationDatabaseService.query(sysDatabaseEntity);
                if (CollectionUtils.isNotEmpty(dataBaseList)){
                    dataSourceCode = dataBaseList.get(0).getSourceCode();
                }
            }
            Map<String, String> headers = entity.getHeaders();
            String formmainTableName = headers.get("formmainTableName");
            String forsonTableName = headers.get("forsonTableName");
            String forsonDetailsTableName = headers.get("forsonDetailsTableName");
            String djlx = headers.get("djlx");
            JSONObject bodyObj =  JSONObject.parseObject(bodys);
            //事件类型
            String eventType = bodyObj.getString("eventType");
            //formApp 表单id
            String formApp = bodyObj.getString("formApp");
            //summaryId
            String summaryId = bodyObj.getString("summaryId");
            JSONObject jsonObject = bodyObj.getJSONObject("businessDataStr");
            if(null!=jsonObject){
                //得到主表和明细表
                JSONObject formmainData = jsonObject.getJSONObject(formmainTableName);
                JSONArray forsonData = jsonObject.getJSONArray(forsonTableName);
                JSONArray forsonDetails = jsonObject.getJSONArray(forsonDetailsTableName);
                String result="";
                //如果单据是销售订单
                if(DjlxEnum.SALES.type.equals(djlx)){
                    result= sales(eventType,formmainData,forsonData,forsonDetails);
                }
                //如果是付款单
                else if(DjlxEnum.PAYMENT.type.equals(djlx)){
                    result= payment(eventType,formmainData,forsonData,forsonDetails);
                }
                if(StrUtil.isNotEmpty(result)){
                    JSONObject jsonStr = JSONObject.parseObject(result);
                    jsonStr.put("formApp",formApp);
                    jsonStr.put("id",formmainData.getString("id"));
                    jsonStr.put("dataSourceCode",dataSourceCode);

                    String jsonString = jsonStr.toJSONString();
                    entity.setBodys(jsonString);
                    return entity;
                }
            }
        }catch (Exception e){
           logger.info("======在调用接口之前，流程参数组装错误:{}===",e.getMessage());
        }
        return null;
    }

    /**
     *
     * @content 销售订单在调用之前参数组装
     * @Param headers:头部信息
     *        eventType：事件类型
     *        summaryId：流程id
     *        dataSourceCode：数据库编码
     *        formmainData：主表数据
     *        forsonData：明细表数据
     * @Return
     * @Author hecan
     * @Date 2024-09-23 17:53
     * **/
    private  String sales(String eventType, JSONObject formmainData, JSONArray forsonData,JSONArray forsonDetailsTableName){
        try {
            //流程结束
           if(ColEventTypeEnum.ONPROCESSFINISHED.getType().equals(eventType)){
               JSONObject jsonMain=new JSONObject();
               jsonMain.put("DocumentType",formmainData.getString("field0001")==null?"SO1":formmainData.getString("field0001"));//单据类型
               jsonMain.put("DocNo",formmainData.getString("field0005"));//单号
               jsonMain.put("OrderBy",formmainData.getString("field0009"));//客户
               jsonMain.put("BusinessDate",formmainData.getString("field0017"));//日期
               jsonMain.put("SaleDepartment",formmainData.getString("field0010"));//部门
               jsonMain.put("Seller",formmainData.getString("field0006"));//业务员
               jsonMain.put("IsPriceIncludeTax",true);//价格是否含税
               jsonMain.put("Memo",formmainData.getString("field0020"));//备注
               jsonMain.put("ConfirmTermCode",formmainData.getString("field0015"));//立账条件
               JSONObject jsonDescFlexField=new JSONObject();
               jsonDescFlexField.put("PubDescSeg1",formmainData.getString("field0003"));//合同号
               jsonDescFlexField.put("PubDescSeg5",formmainData.getString("field0008"));//厂区
               jsonDescFlexField.put("PubDescSeg6",formmainData.getString("field0012"));//贸易方式
               jsonMain.put("DescFlexField",jsonDescFlexField);
               if(CollectionUtils.isNotEmpty(forsonData)){
                   JSONArray jsonArray=new JSONArray();
                   for (Object forsonDatum : forsonData) {
                       JSONObject jsonSOLineDTOList = JSON.parseObject(JSON.toJSONString(forsonDatum));
                       JSONObject jsonDetails=new JSONObject();
                       jsonDetails.put("DocLineNo",jsonSOLineDTOList.getString("field0021"));//行号
                       jsonDetails.put("ItemCode",jsonSOLineDTOList.getString("field0028"));//料号
                       jsonDetails.put("FreeType",jsonSOLineDTOList.getString("field0034"));//免费品类型
                       jsonDetails.put("OrderByQtyTU",jsonSOLineDTOList.getString("field0031"));//数量
                       jsonDetails.put("TU",jsonSOLineDTOList.getString("field0032"));//销售单位编码
                       jsonDetails.put("FinallyPriceTC",jsonSOLineDTOList.getString("field0036"));//最终价
                       jsonDetails.put("TaxSchedule",jsonSOLineDTOList.getString("field0041")==null?"YZ08":jsonSOLineDTOList.getString("field0041"));//税组合
                       jsonDetails.put("ShipTogetherFlag","-1");//成套发货标志
                       jsonDetails.put("Memo",jsonSOLineDTOList.getString("field0042"));//备注
                       jsonDetails.put("RecTermCode",formmainData.getString("field0019"));//收款条件
                       JSONObject jsonDesc=new JSONObject();
                       jsonDesc.put("PrivateDescSeg1",jsonSOLineDTOList.getString("field0022"));//是否处理
                       jsonDesc.put("PrivateDescSeg2",jsonSOLineDTOList.getString("field0026"));//项目明细
                       jsonDesc.put("PrivateDescSeg5",jsonSOLineDTOList.getString("field0025"));//厂区明细id
                       jsonDesc.put("PubDescSeg1",jsonSOLineDTOList.getString("field0023"));//合同号明细
                       jsonDetails.put("DescFlexField",jsonDesc);
                       /*if(CollectionUtils.isNotEmpty(forsonDetailsTableName)){
                           JSONArray jsonArrayLineDTO=new JSONArray();
                           for (Object o : forsonDetailsTableName) {
                               JSONObject jsonSOShiplineDTOListObject = JSON.parseObject(JSON.toJSONString(o));
                               JSONObject jsonObjectShiplineDTO=new JSONObject();
                               jsonObjectShiplineDTO.put("DocSubLineNo",jsonSOLineDTOList.getString(""));//行号
                               jsonObjectShiplineDTO.put("ItemCode",jsonSOLineDTOList.getString("field0028"));//料号
                               jsonObjectShiplineDTO.put("RequireDate",jsonSOShiplineDTOListObject.getString(""));//交期
                               jsonObjectShiplineDTO.put("DemandType",jsonSOShiplineDTOListObject.getString(""));//需求分类
                               jsonObjectShiplineDTO.put("Memo",jsonSOShiplineDTOListObject.getString(""));//备注

                              *//* jsonObjectShiplineDTO.put("SupplySource",jsonSOShiplineDTOListObject.getString(""));//供应来源
                               jsonObjectShiplineDTO.put("DemandTransformType",jsonSOShiplineDTOListObject.getString(""));//供应类型
                               jsonObjectShiplineDTO.put("ShiperOrg",jsonSOShiplineDTOListObject.getString(""));//货主组织
                               jsonObjectShiplineDTO.put("SupplyOrg",jsonSOShiplineDTOListObject.getString(""));//供应组织*//*
                               jsonArrayLineDTO.add(jsonObjectShiplineDTO);
                               jsonDetails.put("SOShiplineDTOList",jsonArrayLineDTO);
                           }
                       }*/

                       JSONArray jsonArrayLineDTO=new JSONArray();
                       JSONObject jsonObjectShiplineDTO=new JSONObject();
                       jsonObjectShiplineDTO.put("DocSubLineNo",jsonSOLineDTOList.getString("field0021"));//行号
                       jsonObjectShiplineDTO.put("ItemCode",jsonSOLineDTOList.getString("field0028"));//料号
                       jsonObjectShiplineDTO.put("RequireDate",jsonSOLineDTOList.getString("field0033"));//交期
                       jsonObjectShiplineDTO.put("DemandType",jsonSOLineDTOList.getString("field0027"));//需求分类
                       jsonObjectShiplineDTO.put("Memo",jsonSOLineDTOList.getString("field0042"));//备注
                       jsonArrayLineDTO.add(jsonObjectShiplineDTO);
                       jsonDetails.put("SOShiplineDTOList",jsonArrayLineDTO);
                       jsonArray.add(jsonDetails);
                       jsonMain.put("SOLineDTOList",jsonArray);
                   }
               }
               String jsonString = jsonMain.toJSONString();
               logger.info("====OA销售订单在流程结束组装的参数为:{}====",jsonString);
               return jsonString;
           }
        }catch (Exception e){
            logger.info("=====销售订单在调用之前参数组装失败：{}====",e.getMessage());
        }
        return null;
    }

    /**
     *
     * @content 付款单在调用之前参数组装
     * @Param headers:头部信息
     *        eventType：事件类型
     *        summaryId：流程id
     *        dataSourceCode：数据库编码
     *        formmainData：主表数据
     *        forsonData：明细表数据
     * @Return
     * @Author hecan
     * @Date 2024-09-23 17:53
     * **/
    private  String payment(String eventType, JSONObject formmainData, JSONArray forsonData,JSONArray forsonDetailsTableName){
        try {
            //流程结束
            if(ColEventTypeEnum.ONPROCESSFINISHED.getType().equals(eventType)){
                JSONObject jsonMain=new JSONObject();
                jsonMain.put("DocNo",formmainData.getString("field0021"));//单号
                jsonMain.put("PCCode",formmainData.getString("field0022")==null?"C001":formmainData.getString("field0022"));//币种
                jsonMain.put("PayDate",formmainData.getString("field0025"));//付款日期
                jsonMain.put("DocumentTypeCode",formmainData.getString("field0018")==null?"APP001":formmainData.getString("field0018"));//单据类型
                jsonMain.put("SuppCode",formmainData.getString("field0030"));//供应商
                jsonMain.put("SuppName",formmainData.getString(""));//供应商名称
                jsonMain.put("SuppSiteCode",formmainData.getString("field0019"));//供应商位置
                jsonMain.put("SuppSiteName",formmainData.getString(""));//供应商位置名称
                jsonMain.put("CustCode",formmainData.getString(""));//客户
                jsonMain.put("CustName",formmainData.getString(""));//客户名称
                jsonMain.put("CustSiteCode",formmainData.getString(""));//客户位置
                jsonMain.put("CustSiteName",formmainData.getString(""));//客户位置名称
                jsonMain.put("PayObjType",formmainData.getString("field0027"));//付款对象
                jsonMain.put("DeptCode",formmainData.getString("field0020"));//部门
                jsonMain.put("ProjectCode",formmainData.getString("field0026"));//项目
                jsonMain.put("SrcBillOrgCode",formmainData.getString("field0024")==null?"1001":formmainData.getString("field0024"));//来源组织
                jsonMain.put("TransactorCode",formmainData.getString("field0029"));//业务员
                jsonMain.put("Note",formmainData.getString(""));//备注
                if(CollectionUtils.isNotEmpty(forsonData)){
                    JSONArray jsonArray=new JSONArray();
                    for (Object forsonDatum : forsonData) {
                        JSONObject jsonPayBillLines = JSON.parseObject(JSON.toJSONString(forsonDatum));
                        JSONObject jsonDetails=new JSONObject();
                        jsonDetails.put("SettlementMethodCode",jsonPayBillLines.getString("field0002"));//结算方式
                        jsonDetails.put("PayBkAccCode",jsonPayBillLines.getString("field0007"));//付款银行账号
                        jsonDetails.put("PayBACCode",jsonPayBillLines.getString("field0008"));//账户币种
                        jsonDetails.put("PayBkSubAccount",jsonPayBillLines.getString("field0009"));//子账户
                        jsonDetails.put("PayACCSettlementFee",jsonPayBillLines.getString("field0005"));//手续费
                        jsonDetails.put("LineNum",jsonPayBillLines.getString("field0001"));//行号
                        jsonDetails.put("CashAccountCode",jsonPayBillLines.getString("field0010"));//现金账号
                        jsonDetails.put("TradeDate",jsonPayBillLines.getString("field0011"));//回单日期
                        jsonDetails.put("Maturity",formmainData.getString("field0035"));//到期日
                        if(CollectionUtils.isNotEmpty(forsonDetailsTableName)){
                            JSONArray jsonArrayUseLine=new JSONArray();
                            for (Object o : forsonDetailsTableName) {
                                JSONObject PayBillUseLine = JSON.parseObject(JSON.toJSONString(o));
                                JSONObject jsonPayBillUse=new JSONObject();
                                jsonPayBillUse.put("PayProperty",PayBillUseLine.getString("field0058")==null?"0":PayBillUseLine.getString("field0058"));//用途
                                jsonPayBillUse.put("Money",PayBillUseLine.getString("field0052"));//金额
                                jsonPayBillUse.put("SettlementFee",jsonPayBillLines.getString("field0005"));//手续费
                                jsonPayBillUse.put("LineNum",PayBillUseLine.getString("field0047"));//行号
                                jsonPayBillUse.put("SuppCode",PayBillUseLine.getString("field0048"));//代付供应商
                                jsonPayBillUse.put("SuppName",PayBillUseLine.getString(""));//代付供应商名称
                                jsonPayBillUse.put("SuppSiteCode",PayBillUseLine.getString(""));//代付供应商位置
                                jsonPayBillUse.put("SuppSiteName",PayBillUseLine.getString(""));//代付供应商位置名称
                                jsonPayBillUse.put("BizOrgCode",formmainData.getString("field0024")==null?"1001":formmainData.getString("field0024"));//业务组织
                                jsonPayBillUse.put("DeptCode",PayBillUseLine.getString("field0067"));//部门
                                jsonPayBillUse.put("TransactorCode",PayBillUseLine.getString("field0068"));//业务员
                                jsonPayBillUse.put("ProjectCode",PayBillUseLine.getString("field0071"));//项目
                                jsonPayBillUse.put("IncExpItemCode",PayBillUseLine.getString("field0072"));//收支项目
                                jsonPayBillUse.put("ItemMasterCode",PayBillUseLine.getString("field0069"));//料品
                                jsonPayBillUse.put("CustCode",PayBillUseLine.getString(""));//代付客户
                                jsonPayBillUse.put("CustName",PayBillUseLine.getString(""));//代付客户名称
                                jsonPayBillUse.put("CustSiteCode",PayBillUseLine.getString(""));//代付客户位置
                                jsonPayBillUse.put("CustSiteName",PayBillUseLine.getString(""));//代付客户位置名称
                                jsonArrayUseLine.add(jsonPayBillUse);
                                jsonDetails.put("PayBillUseLines",jsonArrayUseLine);
                            }
                        }
                        jsonArray.add(jsonDetails);
                        jsonMain.put("PayBillLines",jsonArray);
                    }
                }
                String jsonString = jsonMain.toJSONString();
                logger.info("====OA付款单在流程结束组装的参数为:{}====",jsonString);
                return jsonString;
            }
        }catch (Exception e){
            logger.info("=====付款单在调用之前参数组装失败：{}====",e.getMessage());
        }
        return null;
    }

    private enum DjlxEnum {
        SALES("sales","销售订单"),
        PAYMENT("payment","付款单"),
        ;

        private String type;
        private String name;

        public String getType() {
            return type;
        }

        public void setType(String type) {
            this.type = type;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        DjlxEnum(String type, String name) {
            this.type = type;
            this.name = name;
        }
    }
}
